@@ -1,3 +1,5 @@
+from datetime import datetime
+
 from django import template
 from django.utils.safestring import mark_safe
 
@@ -15,6 +17,7 @@ def shift_status(shift, user):
             AbstractParticipation.States.RESPONSIBLE_REJECTED: "text-danger",
             AbstractParticipation.States.REQUESTED: "text-warning",
             AbstractParticipation.States.CONFIRMED: "text-success",
+            AbstractParticipation.States.GETTING_DISPATCHED: "text-secondary",
         }[participation.state]
         return mark_safe(f'<span class="{color}">{participation.get_state_display()}</span><br>')
     return ""
@@ -43,3 +46,12 @@ def can_decline(shift, user):
 @register.filter(name="decline_errors")
 def decline_errors(shift, user):
     return shift.signup_method.get_decline_errors(user.as_participant())
+
+
+@register.filter(name="confirmed_shifts")
+def confirmed_shifts(user):
+    return (
+        user.get_shifts(with_participation_state_in=[AbstractParticipation.States.CONFIRMED])
+        .filter(end_time__gte=datetime.now())
+        .order_by("start_time")
+    )
