@@ -1,14 +1,19 @@
+import json
+from datetime import datetime, timedelta
+
 from django.contrib import messages
 from django.contrib.auth import get_user_model
 from django.contrib.auth.mixins import LoginRequiredMixin
 from django.contrib.auth.models import Group
 from django.core.exceptions import ValidationError
 from django.db.models import Max, Min
+from django.forms import DateField
 from django.http import HttpResponse
 from django.shortcuts import get_object_or_404, redirect
 from django.urls import reverse, reverse_lazy
 from django.utils import timezone
-from django.utils.timezone import get_default_timezone
+from django.utils.formats import date_format
+from django.utils.timezone import get_current_timezone, get_default_timezone, make_aware
 from django.utils.translation import gettext as _
 from django.views.generic import (
     CreateView,
@@ -19,12 +24,15 @@ from django.views.generic import (
     UpdateView,
     View,
 )
+from django.views.generic.base import TemplateResponseMixin
 from django.views.generic.detail import SingleObjectMixin
-from guardian.shortcuts import get_objects_for_user
+from django.views.generic.edit import FormView
+from guardian.shortcuts import assign_perm, get_objects_for_user, get_users_with_perms
+from recurrence.forms import RecurrenceField
 
-from ephios.event_management.forms import EventForm, ShiftForm
+from ephios.event_management.forms import EventDuplicationForm, EventForm, ShiftForm
 from ephios.event_management.models import Event, Shift
-from ephios.extra.permissions import CustomPermissionRequiredMixin
+from ephios.extra.permissions import CustomPermissionRequiredMixin, get_groups_with_perms
 
 
 class HomeView(LoginRequiredMixin, TemplateView):
@@ -43,6 +51,7 @@ class EventListView(LoginRequiredMixin, ListView):
             )
             .filter(end_time__gte=timezone.now())
             .select_related("type")
+            .order_by("start_time")
         )
 
 
@@ -53,8 +62,7 @@ class EventDetailView(CustomPermissionRequiredMixin, DetailView):
     def get_queryset(self):
         if self.request.user.has_perm("event_management.add_event"):
             return Event.all_objects.all()
-        else:
-            return Event.objects.all()
+        return Event.objects.all()
 
 
 class EventUpdateView(CustomPermissionRequiredMixin, UpdateView):
@@ -118,6 +126,22 @@ class EventDeleteView(CustomPermissionRequiredMixin, DeleteView):
     success_url = reverse_lazy("event_management:event_list")
 
 
+class EventBulkDeleteView(CustomPermissionRequiredMixin, TemplateResponseMixin, View):
+    permission_required = "event_management.delete_event"
+    template_name = "event_management/event_bulk_delete.html"
+
+    def post(self, request, *args, **kwargs):
+        events = Event.objects.filter(pk__in=request.POST.getlist("bulk_action"))
+        if not events:
+            messages.info(request, _("No events were selected for deletion."))
+            return redirect(reverse("event_management:event_list"))
+        if request.POST.get("confirm"):
+            events.delete()
+            messages.info(request, _("The selected events have been deleted."))
+            return redirect(reverse("event_management:event_list"))
+        return self.render_to_response({"events": events})
+
+
 class EventArchiveView(CustomPermissionRequiredMixin, ListView):
     permission_required = "event_management.view_past_event"
     model = Event
@@ -132,9 +156,105 @@ class EventArchiveView(CustomPermissionRequiredMixin, ListView):
             )
             .filter(end_time__lt=timezone.now())
             .select_related("type")
+            .order_by("-start_time")
         )
 
 
+class EventCopyView(CustomPermissionRequiredMixin, SingleObjectMixin, FormView):
+    permission_required = "event_management.add_event"
+    model = Event
+    template_name = "event_management/event_copy.html"
+    form_class = EventDuplicationForm
+
+    def setup(self, request, *args, **kwargs):
+        super().setup(request, *args, **kwargs)
+        self.object = self.get_object()
+
+    def form_valid(self, form):
+        occurrences = form.cleaned_data["recurrence"].between(
+            datetime.now() - timedelta(days=1),
+            datetime.now() + timedelta(days=730),  # allow dates up to two years in the future
+            inc=True,
+            dtstart=datetime.combine(
+                DateField().to_python(self.request.POST["start_date"]), datetime.min.time()
+            ),
+        )
+        for date in occurrences:
+            event = self.get_object()
+            start_date = event.get_start_time().date()
+            shifts = event.shifts.all()
+            event.pk = None
+            event.save()
+            assign_perm(
+                "view_event", get_groups_with_perms(self.get_object(), ["view_event"]), event
+            )
+            assign_perm(
+                "change_event", get_groups_with_perms(self.get_object(), ["change_event"]), event
+            )
+            assign_perm(
+                "change_event",
+                get_users_with_perms(
+                    self.get_object(), only_with_perms_in=["change_event"], with_group_users=False
+                ),
+                event,
+            )
+
+            shifts_to_create = []
+            for shift in shifts:
+                shift.pk = None
+                # shifts on following days should have the same offset from the new date
+                offset = shift.start_time.date() - start_date
+                # shifts ending on the next day should end on the next day to the new date
+                end_offset = shift.end_time.date() - shift.start_time.date()
+                current_tz = get_current_timezone()
+                shift.end_time = make_aware(
+                    datetime.combine(
+                        date.date() + offset + end_offset,
+                        shift.end_time.astimezone(current_tz).time(),
+                    )
+                )
+                shift.meeting_time = make_aware(
+                    datetime.combine(
+                        date.date() + offset, shift.meeting_time.astimezone(current_tz).time()
+                    )
+                )
+                shift.start_time = make_aware(
+                    datetime.combine(
+                        date.date() + offset, shift.start_time.astimezone(current_tz).time()
+                    )
+                )
+                shift.event = event
+                shifts_to_create.append(shift)
+            Shift.objects.bulk_create(shifts_to_create)
+
+        messages.success(self.request, _("Event copied successfully."))
+        return redirect(reverse("event_management:event_list"))
+
+
+class RRuleOccurrenceView(CustomPermissionRequiredMixin, View):
+    permission_required = "event_management.add_event"
+
+    def post(self, *args, **kwargs):
+        try:
+            recurrence = RecurrenceField().clean(self.request.POST["recurrence_string"])
+            return HttpResponse(
+                json.dumps(
+                    recurrence.between(
+                        datetime.now() - timedelta(days=1),
+                        datetime.now()
+                        + timedelta(days=730),  # allow dates up to two years in the future
+                        inc=True,
+                        dtstart=datetime.combine(
+                            DateField().to_python(self.request.POST["dtstart"]), datetime.min.time()
+                        ),
+                    ),
+                    default=lambda obj: date_format(obj, format="SHORT_DATE_FORMAT"),
+                )
+            )
+        except (TypeError, KeyError, ValidationError):
+            return HttpResponse()
+
+
 class ShiftCreateView(CustomPermissionRequiredMixin, TemplateView):
     permission_required = "event_management.change_event"
     template_name = "event_management/shift_form.html"
@@ -152,6 +272,7 @@ class ShiftCreateView(CustomPermissionRequiredMixin, TemplateView):
     def get_context_data(self, **kwargs):
         kwargs.setdefault("event", self.event)
         kwargs.setdefault("form", self.get_shift_form())
+        kwargs.setdefault("configuration_form", "")
         return super().get_context_data(**kwargs)
 
     def post(self, *args, **kwargs):
@@ -162,7 +283,7 @@ class ShiftCreateView(CustomPermissionRequiredMixin, TemplateView):
             signup_method = signup_method_from_slug(self.request.POST["signup_method_slug"])
             configuration_form = signup_method.get_configuration_form(self.request.POST)
         except ValueError as e:
-            raise ValidationError(e)
+            raise ValidationError(e) from e
         if form.is_valid() and configuration_form.is_valid():
             shift = form.save(commit=False)
             shift.event = self.event
@@ -174,25 +295,21 @@ class ShiftCreateView(CustomPermissionRequiredMixin, TemplateView):
                         "event_management:event_createshift", kwargs={"pk": self.kwargs.get("pk")}
                     )
                 )
-            else:
-                try:
-                    self.event.activate()
-                    messages.success(
-                        self.request,
-                        _("The event {title} has been saved.").format(title=self.event.title),
-                    )
-                except ValidationError as e:
-                    messages.error(self.request, e)
-                return redirect(self.event.get_absolute_url())
-        else:
-            return self.render_to_response(
-                self.get_context_data(
-                    form=form,
-                    configuration_form=signup_method.render_configuration_form(
-                        form=configuration_form
-                    ),
+            try:
+                self.event.activate()
+                messages.success(
+                    self.request,
+                    _("The event {title} has been saved.").format(title=self.event.title),
                 )
+            except ValidationError as e:
+                messages.error(self.request, e)
+            return redirect(self.event.get_absolute_url())
+        return self.render_to_response(
+            self.get_context_data(
+                form=form,
+                configuration_form=signup_method.render_configuration_form(form=configuration_form),
             )
+        )
 
 
 class ShiftConfigurationFormView(View):
@@ -242,7 +359,7 @@ class ShiftUpdateView(CustomPermissionRequiredMixin, SingleObjectMixin, Template
             signup_method = signup_method_from_slug(self.request.POST["signup_method_slug"])
             configuration_form = signup_method.get_configuration_form(self.request.POST)
         except ValueError as e:
-            raise ValidationError(e)
+            raise ValidationError(e) from e
         if form.is_valid() and configuration_form.is_valid():
             shift = form.save(commit=False)
             shift.signup_configuration = configuration_form.cleaned_data
@@ -251,20 +368,16 @@ class ShiftUpdateView(CustomPermissionRequiredMixin, SingleObjectMixin, Template
                 return redirect(
                     reverse("event_management:event_createshift", kwargs={"pk": shift.event.pk})
                 )
-            else:
-                messages.success(
-                    self.request, _("The shift {shift} has been saved.").format(shift=shift)
-                )
-                return redirect(self.object.event.get_absolute_url())
-        else:
-            return self.render_to_response(
-                self.get_context_data(
-                    form=form,
-                    configuration_form=signup_method.render_configuration_form(
-                        form=configuration_form
-                    ),
-                )
+            messages.success(
+                self.request, _("The shift {shift} has been saved.").format(shift=shift)
             )
+            return redirect(self.object.event.get_absolute_url())
+        return self.render_to_response(
+            self.get_context_data(
+                form=form,
+                configuration_form=signup_method.render_configuration_form(form=configuration_form),
+            )
+        )
 
 
 class ShiftDeleteView(CustomPermissionRequiredMixin, DeleteView):
@@ -279,8 +392,7 @@ class ShiftDeleteView(CustomPermissionRequiredMixin, DeleteView):
         if self.object.event.shifts.count() == 1:
             messages.error(self.request, _("You cannot delete the last shift!"))
             return redirect(self.object.event.get_absolute_url())
-        else:
-            return super().delete(request, *args, **kwargs)
+        return super().delete(request, *args, **kwargs)
 
     def get_success_url(self):
         messages.success(self.request, _("The shift has been deleted."))
